#!/bin/zsh

# Copyright 2014, 2015, 2020 Paul Hänsch
#
# This file is part of Confetti.
# 
# Confetti is free software: you can redistribute it and/or modify
# it under the terms of the GNU Affero General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
# 
# Confetti is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU Affero General Public License for more details.
# 
# You should have received a copy of the GNU Affero General Public License
# along with Confetti.  If not, see <http://www.gnu.org/licenses/>. 

. "$_EXEC/pdiread.sh"
. "$_EXEC/session_lock.sh"
. "$_EXEC/cgilite/storage.sh"

unset coursefile attfile tempfile

course="$(POST course |PATH)"; course="${course##*/}"
coursefile="$_DATA/ical/$course"
attfile="$_DATA/mappings/attendance"

action="$(POST action)"
newfield="$(POST newfield |grep -m 1 -xE '[A-Z][A-Z0-9-]*')"

if printf '%s\n' "$action" |grep -qxE 'addfield [A-Z][A-Z0-9]*'; then
  newfield="${action##* }"
  action=addfield
fi

if ! tempfile="$(CHECK_SLOCK "$coursefile")"; then
  SET_COOKIE 0 message="NO VALID FILE LOCK"
  REDIRECT "/courses/?e=${course}"
  exit 0
elif [ "$(POST tid)" != "$(transid "$tempfile")" ]; then
  SET_COOKIE 0 message="INVALID TRANSACTION ID"
  REDIRECT "/courses/?e=${course}"
  exit 0
fi

vcf_escape(){
  for each in "$@"; do
    printf %s\\n "$each" \
    | sed -E ':X;$!{N;bX}; s;\r\n;\n;g; s;([;,\\]);\\\1;g; s;\n;\\n;g;'
  done \
  | sed -E ':X;$!{N;bX}; s;\n;\;;g'
}

course="$(pdi_load "$coursefile")"

tzid=$(cat /etc/timezone)

course="$(pdi_update_attrib "$course" DTSTAMP 1 "TZID=${tzid}")"
course="$(pdi_update_value  "$course" DTSTAMP 1 "$(TZ="$tzid" date +%Y%m%dT%H%M%S)")"

dts_year="$(  POST DTS_YEAR    |grep -m1 -xE '[0-9]{4}' || date +%Y)"
dts_month="$( POST DTS_MONTH   |grep -m1 -xE '0[1-9]|1[012]' || date +%m)"
dts_day="$(   POST DTS_DAY     |grep -m1 -xE '0[1-9]|[12][0-9]|3[01]' || date +%d)"
dts_hour="$(  POST DTS_HOUR    |grep -m1 -xE '[0-9]|1[0-9]|2[0-3]' || date +%H)"
dts_minute="$(POST DTS_MINUTE  |grep -m1 -xE '[0-9]|[1-5][0-9]' || date +%M)"
[ ${#dts_hour}   -eq 1 ] && dts_minute="0$dts_hour"
[ ${#dts_minute} -eq 1 ] && dts_minute="0$dts_minute"
DTSTART="${dts_year}${dts_month}${dts_day}T${dts_hour}${dts_minute}00"

course="$(pdi_update_attrib "$course" DTSTART 1 "TZID=${tzid}")"
course="$(pdi_update_value  "$course" DTSTART 1 "$DTSTART")"

rr_int=$(  POST RRULE_INTERVAL |grep -m1 -xE '[0-9]+' || printf 1)
rr_count=$(POST RRULE_COUNT    |grep -m1 -xE '[0-9]+' || printf 1)
rr_freq=$( POST RRULE_FREQ     |grep -m1 -xE 'DAILY|WEEKLY|MONTHLY|YEARLY' || printf MONTHLY)
rr_uy=$(   POST RRULE_UYEAR    |grep -m1 -xE '[0-9]{4}' || date +%Y)
rr_um=$(   POST RRULE_UMONTH   |grep -m1 -xE '[1-9]|1[012]' || date +%m)
rr_ud=$(   POST RRULE_UDAY     |grep -m1 -xE '[1-9]|[12][0-9]|3[01]' || date +%d)
[ ${#rr_um} -eq 1 ] && rr_um="0$rr_um"
[ ${#rr_ud} -eq 1 ] && rr_ud="0$rr_ud"

case $(POST RRULE_LIMIT) in
  COUNT)   RRULE="FREQ=$rr_freq;INTERVAL=$rr_int;COUNT=$rr_count";;
  UNTIL)   RRULE="FREQ=$rr_freq;INTERVAL=$rr_int;UNTIL=${rr_uy}${rr_um}${rr_ud}T000000Z";;
  ETERN|*) RRULE="FREQ=$rr_freq;INTERVAL=$rr_int";;
esac

course="$(pdi_update_value  "$course" RRULE 1 "$RRULE")"

for field in $(POST_KEYS |grep -xE '[A-Z][A-Z0-9-]*'); do
  for cnt in $(seq 1 $(POST_COUNT "$field")); do
    case "$field" in
      *)
         course="$(pdi_update_value "$course" "$field" "$cnt" "$(vcf_escape "$(POST "$field" "$cnt")")")"
        ;;
    esac
done; done

# delete fields, first mark for deletion using delete_key
# this way the field enumeration is preserved during the process
# finally filter marked lines
delete_key="$(randomid)"
for delete in $(POST_KEYS |grep -xE '[A-Z][A-Z0-9-]*_delete_[0-9]+'); do
  f="${delete%%_*}"; c="${delete##*_}";
  [ "$(POST "$delete")" = "true" ] && course="$(pdi_update_value "$course" "$f" "$c" "delete=${delete_key}")"
done
course="$(printf '%s\n' "$course" |sed -E "/^[^:]+:delete=${delete_key}\$/d")"

case "$(POST action)" in
  addfield)
    course="$(pdi_update_value "$course" "$newfield" $(( $(pdi_count "$course" "$newfield") + 1 )) '')"
    printf '%s' "$course" |grep -vx '' >"$tempfile"
    REDIRECT "/courses/?e=${course}"
    ;;
  update)
    if LOCK "$attfile"; then
      grep -F "${course}	" "$attfile" |while read junk card; do
        touch "$_DATA/vcard/${card}"
      done
      sed -i -r "/^${course}	.+\$/d" "$attfile"
      seq 1 $(POST_COUNT attendance) |while read n; do
        printf '%s	%s\n' "$course" "$(POST attendance $n)"
      done >>"$attfile"
      grep -F "${course}	" "$attfile" |while read junk card; do
        touch "$_DATA/vcard/${card}"
      done
      RELEASE "$attfile"
    else
      SET_COOKIE 0 message="COULD NOT UPDATE COURSE MAPPINGS"
    fi

    printf '%s' "$course" |grep -vx '' >"${tempfile}.cp"
    mv "${tempfile}.cp" "$coursefile"
    RELEASE_SLOCK "$coursefile"
    REDIRECT "/courses/#${course}"
    ;;
  cancel)
    RELEASE_SLOCK "$coursefile"
    [ -f "$coursefile" ] \
    && REDIRECT "/courses/#${course}" \
    || REDIRECT "/courses/"
    ;;
  delete)
    rm "$coursefile"
    RELEASE_SLOCK "$coursefile"
    REDIRECT "/courses/"
    ;;
esac
